/*
 *
 * Bertrand Leconte <B.Leconte@mail.dotcom.fr>
 */

#include <unistd.h>
#include <stdio.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <ctype.h>

#include <kapp.h>
#include <kmsgbox.h>
#include <qsocknot.h>
#include <qfileinf.h>

#include "CommandDlg.h"
#include "mCommandDlg.cpp"

#ifdef HAVE_CONFIG
#include <config.h>
#endif

// This is the unit used to separate widgets
#define SEPARATION 10


CommandDlg::CommandDlg(QWidget* parent,	const char* name)
	: QWidget(parent, name)
{
  QPopupMenu *menu;		// for MyMultiLineEdit

  layout1 = new QBoxLayout(this, QBoxLayout::TopToBottom, SEPARATION);
  CHECK_PTR(layout1);
  layout2 = new QBoxLayout(QBoxLayout::LeftToRight, SEPARATION);
  CHECK_PTR(layout2);
  layout1->addLayout(layout2, 0);

  commandLbl1 = new QLabel( this, "label_1" );
  CHECK_PTR(commandLbl1);
  commandLbl1->setText("Host:");
  commandLbl1->setAlignment(AlignLeft|AlignVCenter);
  commandLbl1->setMargin(-1);
  commandLbl1->adjustSize();
  commandLbl1->setFixedSize(commandLbl1->width(), 30);
  layout2->addWidget(commandLbl1);
  
  commandArgs = new QLineEdit( this, "lineedit_1" );
  commandArgs->setMaximumSize(QLayout::unlimited, 30);
  connect(commandArgs, SIGNAL(returnPressed()), 
	  this, SLOT(slotLauchCommand()));
  connect(commandArgs, SIGNAL(textChanged(const char*)), 
	  this, SLOT(slotEnableGoBtn(const char*)));
  commandArgs->setMaxLength(1024);
  commandArgs->setEchoMode(QLineEdit::Normal);
  commandArgs->setFrame(TRUE);
  layout2->addWidget(commandArgs);
  
  layout2->addSpacing(2*SEPARATION);
  
  commandGoBtn = new QPushButton( this, "pushbutton_3" );
  commandGoBtn->setFixedSize(70, 30);
  connect(commandGoBtn, SIGNAL(clicked()),
	  this, SLOT(slotLauchCommand()));
  commandGoBtn->setText( "Go!" );
  commandGoBtn->setEnabled(FALSE);
  isGoBtnEnabled=FALSE;
  commandGoBtn->setFocusPolicy(QWidget::StrongFocus);
  layout2->addWidget(commandGoBtn);

  commandStopBtn = new QPushButton( this, "pushbutton_4" );
  commandStopBtn->setFixedSize(70, 30);
  connect(commandStopBtn, SIGNAL(clicked()), 
	  this, SLOT(slotStopCommand()));
  commandStopBtn->setText("Stop");
  //commandStopBtn->hide();
  commandStopBtn->setEnabled(FALSE);
  //commandStopBtn->setFocusPolicy(QWidget::NoFocus);
  layout2->addWidget(commandStopBtn);
  
  commandTextArea = new MyMultiLineEdit( this, "multilineedit_1" );
  commandTextArea->setReadOnly(TRUE);
  commandTextArea->setFocusPolicy(QWidget::NoFocus);
  menu = commandTextArea->getMenu();
  menu->insertItem("Clear", commandTextArea, SLOT(clear()));
  layout1->addWidget(commandTextArea, 10);

  layout1->activate();

  sn = NULL;			// socket notifier
}

/*
 * Destructor
 */
CommandDlg::~CommandDlg()
{
  slotStopCommand();
}

/**
 * This is called when the tab is selected, so we
 * can manage the focus
 */
void
CommandDlg::tabSelected()
{
  commandArgs->setFocus();
}

/**
 * This is called when the tab is deselected, so we can
 * kill the command if needed
 */
void
CommandDlg::tabDeselected()
{
  slotStopCommand();
}

/**
 * This slot is connected to the lineedit, so we
 * can check if there is something in it to enable
 * the go button.
 */
void
CommandDlg::slotEnableGoBtn(const char *args)
{
  if (strlen(args) == 0) {
    if (isGoBtnEnabled) {
      isGoBtnEnabled = FALSE;
      commandGoBtn->setEnabled(isGoBtnEnabled);
    }
  } else {
    if (!isGoBtnEnabled) {
      isGoBtnEnabled = TRUE;
      commandGoBtn->setEnabled(isGoBtnEnabled);
    }
  }
}


/*
 * This function check if we can call the sub-program
 * (we make some basic checks on length and special characters)
 */
bool
CommandDlg::checkInput(QString *args)
{
  char c;
  int l;
  const char *p;
  
  l = args->length();
  if (l > 128) {
    return(FALSE);
  }
  p = (const char *)*args;
  while ((c = *p++)) {
    if ((!isalnum(c)) 
	&& (c!='.') && (c != '-') && (c != '/') && (c != '_')) {
      return(FALSE);
    }
  }
  return(TRUE);
}

/**
 * build the command line from widgets
 */
void
CommandDlg::buildCommandLine(QString args)
{
  childProcess.wantCommunication(TRUE);
  childProcess.clearArguments();
  childProcess << (const char *)args;
}


/*
 * this is called from the Go button
 */
void
CommandDlg::slotLauchCommand()
{
  QString str;
  QString args;

  if (childProcess.isRunning()) {
    return;
  }

  args = commandArgs->text();
  if (strlen(args) == 0) {
    // nothing to do (this should not be possible)
  } else {

    // Check the input
    if (!checkInput(&args)) {
      //warning("input not valid");
      commandArgs->selectAll();
      KApplication::beep();
      return;
    }

    // Install the "Stop" button, and hide "Go!"
    //commandGoBtn->hide();
    //commandStopBtn->show();
    commandGoBtn->setEnabled(FALSE);
    commandStopBtn->setEnabled(TRUE);
    //commandGoBtn->setFocusPolicy(QWidget::NoFocus);
    //commandStopBtn->setFocusPolicy(QWidget::StrongFocus);
    // Install waitCursor
    KApplication::setOverrideCursor(waitCursor);

    // separate commands with CR/LF
    if (commandTextArea->numLines() > 1) {
      commandTextArea->append("");
    }

    //  Process creation
    buildCommandLine(args);
    connect(&childProcess, SIGNAL(processExited(KProcess *)), 
	    SLOT(slotProcessDead(KProcess *)));

    if (!childProcess.start()) {
      // Process not started
      slotProcessDead(NULL);
      return;
    }
    sn = new QSocketNotifier(childProcess.getSocket(), 
			     QSocketNotifier::Read, this);
    CHECK_PTR(sn);
    connect(sn, SIGNAL(activated(int)), 
	    this, SLOT(slotCmdStdout()));
    
  }
}


/*
 * This is called by the Stop button
 */
void
CommandDlg::slotStopCommand()
{
  if (childProcess.isRunning()) {
    childProcess.stop();
  }
}


/*
 * Clean up all the things after the death of the command 
 * (gop button, cursor, ...)
 */
void
CommandDlg::slotProcessDead(KProcess *)
{
  // socket notifier
  disconnect(sn, SIGNAL(activated(int)), 
	     this, SLOT(slotCmdStdout()));
  delete sn;
  
  // disconnect KProcess
  disconnect(&childProcess, SIGNAL(processExited(KProcess *)), 
	     this, SLOT(slotProcessDead(KProcess *)));
  
  // put back the "Go!" button up
  //commandStopBtn->hide();
  //commandGoBtn->show();
  //commandGoBtn->setFocusPolicy(QWidget::StrongFocus);
  //commandStopBtn->setFocusPolicy(QWidget::NoFocus);
  commandGoBtn->setEnabled(TRUE);
  commandStopBtn->setEnabled(FALSE);
  KApplication::restoreOverrideCursor();
  
  // to be ready for a new command
  commandArgs->selectAll();
  commandArgs->setFocus();
}

/*
 * Read the output of the command on a socket and append
 * it to the multilineedit.
 *
 * This is called by the main select loop in Qt (QSocketNotifier)
 */
void
CommandDlg::slotCmdStdout()
{
  char buf[1024];
  int  rc, line, col;
  char *p;

  // Read the output data of our child
  rc = read(childProcess.getSocket(), buf, 1023);
  if (rc > 0) {
    buf[rc] = 0;		// mark eot
    //debug("text = \"%s\"", buf);

    // goto end of data
    line = QMAX(commandTextArea->numLines() - 1, 0);
    p = (char*)commandTextArea->textLine(line);
    col = 0;
    if (p != NULL) {
      col = strlen(p);
    }
    commandTextArea->setCursorPosition(line, col);

    // and insert text here
    commandTextArea->insertAt(buf, line, col);
    commandTextArea->setCursorPosition(commandTextArea->numLines(), 0);
  }
}

/*
 * set the command path and name to use if the class
 * is not derived. (If the class is derived, the method
 * executeCommand will put the name directly.)
 */
void
CommandDlg::setCommand(const char *cmdString)
{
  QFileInfo  fi;
  KConfig   *kc;
  QString    s;
  
  kc = KApplication::getKApplication()->getConfig();

  s = cmdString;
  fi.setFile(cmdString);
  if (fi.isRelative()) {
    // check in configfile
    kc->setGroup(fi.fileName());
    if (kc->hasKey("path")) {
      s = kc->readEntry("path");
    }
  }
  childProcess.assignProcess(s);
}
