//
//  KPROCESS -- A class for handling child processes in KDE without
//  having to take care of Un*x specific implementation details
//
//  version 0.0.1, July 13th 1997
//
//  (C) Christian Czezatke
//  e9025461@student.tuwien.ac.at
//

#include "kprocess.h"
#define _MAY_INCLUDE_KPROCESSCONTROLLER_
#include "kprocctrl.h"

#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <fcntl.h>

#include "mkprocess.cpp"



KProcess::KProcess()
{ 
 arguments.setAutoDelete(TRUE);

 if (NULL == theKProcessController) {
   theKProcessController= new KProcessController();
   CHECK_PTR(theKProcessController);
 }

 run_mode = notify_on_exit;
 runs = FALSE;
 event_pending = FALSE;
 process = NULL;
 pid = 0;
 status = 0;
 communication = FALSE;

 // TODO: block sigchld while inserting into the process list
 theKProcessController->processList->append(this);
}

KProcess::~KProcess()
{
  // destroying the KProcess instance sends a SIGKILL to the
  // child process (if it is running) after removing it from the
  // list of valid processes (if the process is not started as
  // "dont_care")

  theKProcessController->processList->remove(this);
  // this must happen before we kill the child
  // TODO: block the signal while removing the current process from the process list

  if (runs && (run_mode != dont_care))
    stop(SIGKILL);
}


void KProcess::assignProcess(const char *proc)
{
 if (NULL != process)
    free(process);

 if (NULL == proc)
    process = NULL;
 else {
    process = strdup(proc);
    CHECK_PTR(process);
 }
}

KProcess &KProcess::operator<<(const char *arg)
{
 char *new_arg= strdup(arg);

 CHECK_PTR(new_arg);
 arguments.append(new_arg);
 return *this;
}


bool KProcess::start(KRunMode runmode)
{
 uint i;
 uint n = arguments.count();
 char **arglist;
 int newfd;

 theKProcessController->slotDoHousekeeping();
 // bring the "running" state of all child processes up to date
 // and deliver all eventually pending events caused by child processes

 if (runs) {
   return FALSE;  // cannot start a process that is already running
 }
 run_mode = runmode; 
 status = 0;

 if (NULL == process) {
     // cannot start any process before an executable has been assigned to the process
     return FALSE;
 }

 arglist = (char **)malloc( (n+2)*sizeof(char *));
 CHECK_PTR(arglist);

 arglist[0]= process;
 for (i=0; i < n; i++)
   arglist[i+1] = arguments.at(i);
 arglist[n+1]= NULL;


 // Create sockets for communications
 if (communication) {
   int rc;

   rc = socketpair(AF_UNIX, SOCK_STREAM, 0, sockfd);
   if (rc < 0) {
     // We had an error.
     warning("can't socketpair(): %s", strerror(errno));
     return(FALSE);
   }
 }

 runs = TRUE;
 pid = fork();

 if (0 == pid) {
   // The child process

   if (communication) {
     newfd = dup(1);

     dup2(sockfd[1], 0);
     dup2(sockfd[1], 1);
     dup2(sockfd[1], 2);
     close(sockfd[0]);

     struct linger so;
     so.l_onoff = 1;
     so.l_linger = 150;
     setsockopt(sockfd[1], SOL_SOCKET, SO_LINGER, 
		(void *)&so, sizeof(so));
   }
#if 0
   write(newfd, "> ", 2);
   write(newfd, process, strlen(process));
   write(newfd, " ", 1);
   for (i=0; i < n; i++) {
     write(newfd, arguments.at(i), strlen(arguments.at(i)));
     write(newfd, " ", 1);
   }
   write(newfd, "\n", 1);
#endif
   execvp(process, arglist);
   exit(-1);
 } else if (-1 == pid) {
   // forking failed
   free(arglist);
   runs = FALSE;
   return FALSE;
 } else {
   if (communication) {
     int rc;
     close(sockfd[1]);
     rc = fcntl(sockfd[0], F_SETFL, O_NONBLOCK);
     if (rc <0) {
       warning("fcntl O_NONBLOCK: %s", strerror(errno));
     }
   }

   // the parent continues here
   switch (run_mode) {
   case dont_care:
     break;
   case notify_on_exit:
     // start timer, if none is running
     theKProcessController->startTimer();
     break;
   case block:
     char timer_was_running = theKProcessController->isTimerActive();
     theKProcessController->stopTimer();
     while (!event_pending)
       ::select(0,NULL,NULL,NULL,NULL);
     if (timer_was_running)
       theKProcessController->startTimer();
     break;
   }

 }

 free(arglist);    
 return TRUE;
}

bool KProcess::isRunning()
{
  if (event_pending)
    runs = FALSE;
  return runs;
}


bool KProcess::stop(int signo)
{
 bool rv=FALSE;

 if (0 != pid)
    rv= (-1 != kill(pid, signo));
 // probably store errno somewhere...
 return rv;
}


void KProcess::clearArguments()
{
  if (NULL != arguments.first()) {
    while (arguments.remove())
      ;
  }
}


pid_t KProcess::getPid()
{
  return pid;
}


bool KProcess::didExitNormally()
{
  return (pid != 0) && (!runs) && (WIFEXITED(status));
}


int KProcess::exitStatus()
{
  return WEXITSTATUS(status);
}


void KProcess::emitProcessExited()
{
  emit processExited(this);
}


/**
 * 
 */
int
KProcess::getSocket()
{
  if (communication) {
    return sockfd[0];
  } else {
    return(0);
  }
}

/**
 *
 */
void
KProcess::wantCommunication(bool flag)
{
  if (!runs) {
    communication = flag;
  }
}
