//
//  KPROCESS -- A class for handling child processes in KDE without
//  having to take care of Un*x specific implementation details
//
//  version 0.0.1, July 13th 1997
//
//  (C) Christian Czezatke
//  e9025461@student.tuwien.ac.at
//

#ifndef __kprocess_h__
#define __kprocess_h__

#include <sys/types.h> // for pid_t
#include <sys/wait.h>
#include <signal.h>
#include <unistd.h>
#include <qstrlist.h>
#include <qobject.h>


/* various run--modes for a child process */
typedef enum { dont_care, notify_on_exit, block } KRunMode;

/*
  KProcess -- A class for invoking Child processes from within KDE applications

  This class allows a KDE application to start child processes without having
  to worry about UN*X signal handling issues and zombie process reaping

  Basically, this class distinguishes three different ways of running child processes:

   +) "dont_care" -- The child process is invoked and both the child
      process and the parent process continue concurrently. 

      Starting a "dont_care" child process means that the application is
      not interested in any notification to determine whether the child process
      has already exited or not.

   +) "notify_on_exit" -- The child process is invoked both the
      chid and the parent process run concurrently.

      When the child process exits, the KProcess instance
      corresponding to it emits the Qt Signal "processExited".

      Since this signal is _not_ emitted from within a UN*X
      signal handler, arbitrary function calls can be made.

   +) "block" -- The child process starts and the parent process is suspended
      until the child process exits. (_Really_ not recommended for programs
      with a GUI)

  KProcess also provides several functions for determining the exit status
  and the pid of the child process it represents. 

  Furthermore it is possible to supply command-line arguments to the process
  in a clean fashion (no null -- terminated stringlists and such...)

  A small usage example:

  KProcess proc;

    proc.assignProcess("my_executable");
    proc << "These" << "are" << "the" << "command" << "line" << "args";
    QApplication::connect(&proc, SIGNAL(processExited(KProcess *)), 
                          pointer_to_my_object, SLOT(my_objects_slot));
    proc.start();
  
  This will start "my_executable" with the commandline arguments "These"...

  When the child process exits, the respective slot command(s) according to
  the Qt--connect statements will be invoked.
*/


class KProcess : public QObject
{
 Q_OBJECT;

 public:

   KProcess();
   virtual ~KProcess();
   /*
     If the process is running when the destructor for this class
     is called, the child process is killed with a SIGKILL, but
     only if the run mode is not of type "dont_care". --
     Processes started as "dont_care" keep running anyway...
   */
 
   void assignProcess(const char *proc);
   /* Sets the executable to be started with this KProcess object */

   KProcess &operator<<(const char *arg);
   /* sets the command line argument list for this process */

   void clearArguments();
   /* clears the command line argument list for this process */

   bool start(KRunMode  runmode = notify_on_exit);
   /* starts up the process. -- For a detailed description of the
      various run modes see the general comment on "KProcess"

      returns TRUE if the process was started successfully.
      A return value of FALSE indicates that :
        +) the process is already running
        or
	+) the starting of the process failed (could not fork)
   */

   bool stop(int signo = SIGTERM);
   /*
     stops the process (by sending a SIGTERM to it). -- You may send other
     signals too of course... ;-) ) 

     returns TRUE if the signal could be delivered successfully
   */
   bool isRunning();
   /*
     returns TRUE if the process is (still) considered to be running
   */
   
   pid_t getPid();
   bool didExitNormally();
   int  exitStatus(); // only valid if didExitNormally()
   
   /*
     These functions allow to retrieve information about the running
     process. 
       getPid -> returns the process id of the process (can be called
         even _after_ the process has exited. -- Calling it before
	 executing the process will return 0
       didExitNormally -> The process exited "volunatarily", ie: it
         was not killed by any signal.
       exitStatus -> The return value that the process has delivered.
         Of course this is only a valid value for processes that
	 have finished and did exit normally.
    */

							
   /**
    * return a socket that can be used to communicate with the child
    */
   int getSocket();

   /**
    * say if KProcess have to create a socket to communicate with the child
    */
   void wantCommunication(bool);
   
 signals:

   void processExited(KProcess *proc);
   /*
     This signal gets emitted after the process has terminated when the process 
     was run in the "notify_on_exit" mode (=default option to "start"). 
   */     
private:
   friend class KProcessController;

   void emitProcessExited();

   QStrList arguments;
   KRunMode run_mode;
   bool runs;
   bool event_pending;
   char *process;
   pid_t pid;
   int status;

   int  sockfd[2];		/* socket descriptors */
   bool communication;		/* communication with the child? */
};



#endif
